Attribute VB_Name = "modResMenu"
Option Explicit

' Used for 'LOWORD'/'HIWORD' macros.
Private Declare Sub CopyMemory Lib "kernel32.dll" Alias "RtlMoveMemory" (ByRef Destination As Any, ByRef Source As Any, ByVal Length As Long)

' Subclassing functions.
Private Declare Function CallWindowProc Lib "user32.dll" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hWnd As Long, ByVal MSG As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function GetWindowLong Lib "user32.dll" Alias "GetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long) As Long
Private Declare Function SetWindowLong Lib "user32.dll" Alias "SetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long

' Menu functions.
Private Declare Function LoadMenu Lib "user32.dll" Alias "LoadMenuA" (ByVal hInstance As Long, ByVal lpString As String) As Long
Private Declare Function SetMenu Lib "user32.dll" (ByVal hWnd As Long, ByVal hMenu As Long) As Long
Private Declare Function DestroyMenu Lib "user32.dll" (ByVal hMenu As Long) As Long

' Accelerator functions.
Private Declare Function LoadAccelerators Lib "user32.dll" Alias "LoadAcceleratorsA" (ByVal hInstance As Long, ByVal lpTableName As String) As Long
Private Declare Function TranslateAccelerator Lib "user32.dll" Alias "TranslateAcceleratorA" (ByVal hWnd As Long, ByVal hAccTable As Long, ByRef lpMsg As MSG) As Long

' Message functions.
Private Declare Function GetMessage Lib "user32.dll" Alias "GetMessageA" (ByRef lpMsg As MSG, ByVal hWnd As Long, ByVal wMsgFilterMin As Long, ByVal wMsgFilterMax As Long) As Long
Private Declare Function TranslateMessage Lib "user32.dll" (ByRef lpMsg As MSG) As Long
Private Declare Function DispatchMessage Lib "user32.dll" Alias "DispatchMessageA" (ByRef lpMsg As MSG) As Long
Private Declare Sub PostQuitMessage Lib "user32.dll" (ByVal nExitCode As Long)

Private Const GWL_WNDPROC As Long = (-4&)
Private Const WM_SYSCOMMAND As Long = &H112&
Private Const WM_COMMAND As Long = &H111&
Private Const WM_InitializeSubclassingMENU As Long = &H116&
Private Const WM_DESTROY As Long = &H2&
Private Const WM_QUIT As Long = &H12&

Private Type POINTAPI
    x As Long
    y As Long
End Type

Private Type MSG
    hWnd As Long
    Message As Long
    wParam As Long
    lParam As Long
    time As Long
    pt As POINTAPI
End Type

' IDs der Meneintrge.
Private Const ID_FILE_OPEN As Long = 4001&
Private Const ID_FILE_QUIT As Long = 4002&
Private Const ID_FILE_OPTIONS_GENERAL As Long = 4003&
Private Const ID_FILE_OPTIONS_CONNECTION As Long = 4004&
Private Const ID_HELP_ABOUT As Long = 4005&

' Handle der originalen Fensterprozedur.
Private m_pPrevWndProc As Long

' Dieses Programm wird nicht ber das Formular gestartet, es wird eine 'Main'-Prozedur verwendet, da hier
' die Nachrichtenschleife abgelegt wird. Hier werden auch die Daten aus der Ressourcendatei geladen
' und anschliessend wieder der Speicher freigegeben.
Private Sub Main()
    Call Load(FMain)
    
    ' Subclassing des Formular initialisieren.
    Call InitializeSubclassing(FMain.hWnd)
    
    ' Men aus Ressourcendatei laden.
    Dim hMenu As Long
    hMenu = LoadMenu(App.hInstance, "IDR_MENUMAIN")
    
    ' Men auf Formular setzen.
    Call SetMenu(FMain.hWnd, hMenu)
    
    ' Formular anzeigen.
    Call FMain.Show
    
    ' Laden der Tabelle mit den Zugriffstasten (funktioniert nicht in der IDE).
    Dim hAccelTable As Long
    hAccelTable = LoadAccelerators(App.hInstance, "IDR_MAINFRAME")
    
    ' Hauptnachrichtenschleife der Anwendung. Hier werden alle Meldungen bersetzt und an die
    ' Fensterprozedur weitergegeben.
    Dim Message As MSG
    
    ' Diese Schleife wird erst beim Beenden der Anwendung verlassen.
    Do While GetMessage(Message, 0&, 0&, 0&)    ' Warten, solange keine 'WM_QUIT' gefunden wird.
        
        ' Meldung nur weiter verarbeiten, wenn es sich nicht um eine Accelerator-Message gehandelt
        ' hat. Hier wird die Accelerator-Message, falls eine vorbeikommt, in 'WM_COMMAND' oder
        ' 'WM_SYSCOMMAND' bersetzt und anschliessend mittels 'DispatchMessage' an die Fensterprozedur
        ' weitergereicht.
        If TranslateAccelerator(FMain.hWnd, hAccelTable, Message) = 0& Then
            Call TranslateMessage(Message)
        End If
        Call DispatchMessage(Message)
    Loop
    
    ' Unload created objects.
    Call TerminateSubclassing(FMain.hWnd)
    Call DestroyMenu(hMenu)     ' Destroy the menu.
    Call Unload(FMain)
End Sub

' Subclassing starten.
Public Sub InitializeSubclassing(ByVal hWnd As Long)
    m_pPrevWndProc = SetWindowLong(hWnd, GWL_WNDPROC, AddressOf SubWndProc)
End Sub

' Subclassing beenden.
Public Sub TerminateSubclassing(ByVal hWnd As Long)
    Call SetWindowLong(hWnd, GWL_WNDPROC, m_pPrevWndProc)
End Sub

Private Function SubWndProc(ByVal hWnd As Long, ByVal MSG As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    Select Case MSG
        Case WM_COMMAND
            Dim n As Long, i As Long
            n = wParam
            
            ' Die folgende Schleife wre nicht erforderlich, um aber keine doppelte Auswertung zu
            ' bentigen, wurde dies hier so gelst. Eventuell knnen diese Meldungen auch
            ' "geschluckt" werden, da es aber auch so funktioniert und dadurch einfacher ist, wurde
            ' darauf verzichtet.
            Do
                If i = 1 Then i = 2
                Select Case n
                    Case ID_FILE_OPEN
                        MsgBox "Open file."
                    Case ID_FILE_QUIT
                        Call PostQuitMessage(0)
                    Case ID_HELP_ABOUT
                        FAbout.Show vbModal, FMain
                    Case ID_FILE_OPTIONS_GENERAL
                        MsgBox "General options."
                    Case ID_FILE_OPTIONS_CONNECTION
                        MsgBox "Connection settings."
                    Case Else
                        If i <> 1 Then
                            If HIWORD(wParam) = 1 Then
                                n = LOWORD(wParam)
                                i = 1
                            End If
                        End If
                End Select
            Loop While i = 1
        Case WM_DESTROY
            
            ' Senden einer 'WM_QUIT'-Message an die Nachrichtenwarteschlage.
            Call PostQuitMessage(0&)
    End Select
    SubWndProc = CallWindowProc(m_pPrevWndProc, hWnd, MSG, wParam, lParam)
End Function

' Returns the low 16-bit integer from a 32-bit long integer.
Public Function LOWORD(ByVal dwValue As Long) As Integer
    Call CopyMemory(LOWORD, dwValue, 2&)
End Function

' Returns the high 16-bit integer from a 32-bit long integer.
Public Function HIWORD(ByVal dwValue As Long) As Integer
    Call CopyMemory(HIWORD, ByVal VarPtr(dwValue) + 2, 2&)
End Function
